#include "StdAfx.h"

#include "Convertor.h"

//=====================================================================================//
//                                                                                     //
//=====================================================================================//
void readCmdLineOptions(std::map<std::string,std::string>& options,int argc,char* argv[]);
int convertAvi2Flv(const std::string& input_file_name,const std::string& output_file_name);
void typeMediaInfo(const MediaDetector& md);
void typeAudioFormat(unsigned int format);
void typeError(const std::string& msg);
void typeHelpInfo();

//=====================================================================================//
//                                struct CoInitializer                                 //
//=====================================================================================//
struct CoInitializer
{
	bool m_result;
	CoInitializer() : m_result(!FAILED(CoInitialize(0))) {}
	~CoInitializer() { if(m_result) CoUninitialize(); }
	operator bool () { return m_result; }
};

//=====================================================================================//
//                             class MainConvertorObserver                             //
//=====================================================================================//
class MainConvertorObserver : public ConvertorObserver
{
private:

	unsigned int m_duration;

public:
	MainConvertorObserver(unsigned int duration);
	///	process progress of converting
	void onProgress(unsigned int timestamp);

};

MainConvertorObserver::MainConvertorObserver(unsigned int duration)
: m_duration(duration)
{
}
//	process progress of converting
void MainConvertorObserver::onProgress(unsigned int timestamp)
{
	const float percent = static_cast<float>(timestamp)/m_duration*100;
	std::cout << static_cast<unsigned int>(percent) << "%\r";
}

//=====================================================================================//
//                                     int main()                                      //
//=====================================================================================//
int main(int argc,char* argv[])
{
	typedef std::map<std::string,std::string> options_t;

	std::string output_file_name;
	std::string input_file_name;
	options_t options;

	readCmdLineOptions(options,argc,argv);

	if(argc > 1)
	{
		input_file_name = argv[1];
		output_file_name = input_file_name+".flv";

		//	check ouput file name parameter
		if((argc > 2) && (argv[2][0] != '-'))
		{
			output_file_name = argv[2];
		}

		//	check '-key' parameter
		options_t::iterator ikey = options.find("-key");
		if(ikey != options.end())
		{
			//	set secure key
			ftSetContextKey(ikey->second.c_str());
		}

		return convertAvi2Flv(input_file_name,output_file_name);
	}
	else
	{
		typeHelpInfo();
	}

	return EXIT_SUCCESS;
}

//=====================================================================================//
//                              void readCmdLineOptions()                              //
//=====================================================================================//
void readCmdLineOptions(std::map<std::string,std::string>& options,int argc,char* argv[])
{
	typedef std::map<std::string,std::string> options_t;
	options_t::iterator ikey = options.end();

	for(int i=1;i<argc;++i)
	{
		if(argv[i][0] == '-')
		{//	 
			ikey = options.insert(options_t::value_type(argv[i],"")).first;
		}
		else
		{
			if(ikey != options.end())
			{
				ikey->second = argv[i];
				ikey = options.end();
			}
		}
	}
}

//=====================================================================================//
//                                int convertAvi2Flv()                                 //
//=====================================================================================//
int convertAvi2Flv(const std::string& input_file_name,const std::string& output_file_name)
{
	CoInitializer coinitializer;

	if(coinitializer)
	{
		try
		{
			Convertor convertor(input_file_name);
			typeMediaInfo(convertor.getMediaDetector());

			MainConvertorObserver observer(convertor.getMediaDetector().getDuration());

			convertor.setObserver(&observer);
			convertor.convertToFlv(output_file_name);
			observer.onProgress(convertor.getMediaDetector().getDuration());

			std::cout << std::endl;
			std::cout << std::endl << "...done" << std::endl;
		}
		catch(std::exception& e)
		{
			typeError(e.what());
			return EXIT_FAILURE;
		}
	}

	return EXIT_SUCCESS;
}

//=====================================================================================//
//                                void typeMediaInfo()                                 //
//=====================================================================================//
void typeMediaInfo(const MediaDetector& md)
{
	const MDVideoDataFormat* video = md.getVideoDataFormat();
	const MDAudioDataFormat* audio = md.getAudioDataFormat();

	std::cout << "Video stream: ";
	if(video)
	{
		std::cout << video->getWidth() << "x" << video->getHeight() << "x" << video->getBitCount();
		std::cout << " [" << video->getFrameRate() << "] frames/second" << std::endl;
	}
	else std::cout << "is absent" << std::endl;

	std::cout << "Audio stream: ";
	if(audio)
	{
		std::cout << "format ["; typeAudioFormat(audio->getFormatType()); std::cout << "] ";
		std::cout << "rate [" << audio->getSamplesPerSec() << "] ";
		std::cout << "bps [" << audio->getBitsPerSample() << "] ";
		std::cout << "channels [" << audio->getChannels() << "] ";
		std::cout << std::endl;
	}
	else std::cout << "is absent" << std::endl;
}

//=====================================================================================//
//                               void typeAudioFormat()                                //
//=====================================================================================//
void typeAudioFormat(unsigned int format)
{
	switch(format)
	{
	case WAVE_FORMAT_UNKNOWN:				std::cout << "Unknown"; break;
	case WAVE_FORMAT_MPEGLAYER3:			std::cout << "ISO/MPEG Layer3 Format"; break;
	case WAVE_FORMAT_PCM:					std::cout << "PCM"; break;
	case WAVE_FORMAT_DSPGROUP_TRUESPEECH:	std::cout << "DSP Group TrueSpeech"; break;
	default:								std::cout << "Not At List [" << format << "]";
	}
}

//=====================================================================================//
//                                  void typeError()                                   //
//=====================================================================================//
void typeError(const std::string& msg)
{
	std::cout << "Error: " << msg << std::endl;
}

//=====================================================================================//
//                                 void typeHelpInfo()                                 //
//=====================================================================================//
void typeHelpInfo()
{
	std::cout << "usage: avi2flv <input.avi> [output.flv]" << std::endl;
}
